class SettingsController < ApplicationController

  before_action :plan_exists, only: [:update_plan]

  def help
    @user = current_user
  end

  def settings
    @user = current_user
  end

  def sharing
    @user = current_user
  end

  def account
    @user = current_user
  end

  def feeds
    @user = current_user
    @subscriptions = @user.subscriptions.select('subscriptions.*, feeds.title AS original_title, feeds.last_published_entry AS last_published_entry, feeds.feed_url, feeds.site_url').joins("INNER JOIN feeds ON subscriptions.feed_id = feeds.id AND subscriptions.user_id = #{@user.id}")

    feed_ids = @subscriptions.map {|subscription| subscription.feed_id}
    entry_counts = Rails.cache.fetch("#{@user.id}:entry_counts", expires_in: 24.hours) { get_entry_counts(feed_ids) }

    @subscriptions = @subscriptions.map {|subscription|
      subscription.entries_count = entry_counts[subscription.feed_id]
      if subscription.title
        subscription.title = subscription.title
      elsif subscription.original_title
        subscription.title = subscription.original_title
      else
        subscription.title = '(No title)'
      end
      subscription
    }

    @subscriptions = @subscriptions.sort_by {|subscription| subscription.title.downcase}
  end

  def billing
    @user = current_user
    @next_payment = @user.billing_events.where(event_type: 'invoice.payment_succeeded')
    @next_payment = @next_payment.to_a.sort_by {|next_payment| -next_payment.details.data.object.date }
    if @next_payment.present?
      @next_payment.first.details.data.object.lines.data.each do |event|
        event = event.to_hash
        if event[:type] && event[:type] == 'subscription'
          @next_payment_date = Time.at(event[:period].end).utc.to_datetime
        end
      end
    end
    @billing_events = @user.billing_events.where(event_type: 'charge.succeeded')
    @billing_events = @billing_events.to_a.sort_by {|billing_event| -billing_event.details.data.object.created }
    if @user.plan.stripe_id == 'trial'
      @plans = Plan.where(stripe_id: ['basic-monthly-2', 'basic-yearly-2']).order('id DESC')
    elsif @user.plan.stripe_id == 'free'
      @plans = Plan.where(price_tier: @user.plan.price_tier)
    else
      @plans = Plan.where(price_tier: @user.plan.price_tier).where.not(stripe_id: ['free', 'trial'])
    end
  end

  def import_export
    @user = current_user
    @uploader = Import.new.upload
    @uploader.success_action_redirect = settings_import_export_url

    if params[:key]
      @import = Import.new(key: params[:key], user: @user)

      if @import.save
        redirect_to settings_import_export_url, notice: 'Import has started.'
      else
        @messages = @import.errors.full_messages
        flash[:error] = render_to_string partial: "shared/messages"
        redirect_to settings_import_export_url
      end
    end
  end

  def mark_favicon_complete
    session[:favicon_complete] = true
    render nothing: true
  end

  def update_plan
    plan = Plan.find(params[:plan])
    @user = current_user
    @user.plan = plan
    @user.save
    customer = Stripe::Customer.retrieve(@user.customer_id)
    customer.update_subscription(plan: plan.stripe_id)
    redirect_to settings_billing_path
  end

  def update_credit_card
    @user = current_user
    @user.stripe_token = params[:stripe_token]

    respond_to do |format|
      if @user.save
        format.html { redirect_to settings_billing_url, notice: 'Your credit card has been updated.' }
      else
        format.html { redirect_to settings_billing_url, alert: @user.errors.messages[:base].join(' ') }
      end
    end
  end

  def settings_update
    @user = current_user
    @user.attributes = user_settings_params
    if @user.save
      if params[:redirect_to]
        redirect_to params[:redirect_to], notice: 'Settings updated.'
      else
        redirect_to settings_path, notice: 'Settings updated.'
      end
    else
      redirect_to settings_path, alert: @user.errors.full_messages.join('. ') + '.'
    end
  end

  def view_settings_update
    @user = current_user
    if params[:tag_visibility]
      tag_id = params[:tag].to_s
      if @user.tag_visibility[params[:tag]].blank?
        @user.update_tag_visibility(tag_id, true)
      else
        @user.update_tag_visibility(tag_id, false)
      end
    end

    if params[:column_widths]
      session[:column_widths] ||= {}
      session[:column_widths][params[:column]] = params[:width]
    end
    render nothing: true
  end

  def font_increase
    change_font_size('increase')
  end

  def font_decrease
    change_font_size('decrease')
  end

  def font
    @user = current_user
    if Feedbin::Application.config.fonts.has_value?(params[:font])
      @user.font = params[:font]
      @user.save
    end
    render nothing: true
  end

  def theme
    @user = current_user
    if @user.theme.blank? || @user.theme == 'day'
      @user.theme = 'night'
    else
      @user.theme = 'day'
    end
    @user.save
    render nothing: true
  end

  def entry_width
    @user = current_user
    if @user.entry_width.blank?
      new_width = 'fluid'
    else
      new_width = ''
    end
    @user.entry_width = new_width
    @user.save
    render nothing: true
  end

  private

  def change_font_size(direction)
    @user = current_user

    current_font_size = @user.font_size.try(:to_i) || 5
    if direction == 'increase'
      new_font_size = current_font_size + 1
    else
      new_font_size = current_font_size - 1
    end

    if Feedbin::Application.config.font_sizes[new_font_size] && new_font_size >= 0
      @user.font_size = new_font_size
      @user.save
    end

    render nothing: true
  end

  def plan_exists
    render_404 unless Plan.exists?(params[:plan].to_i)
  end

  def user_settings_params
    params.require(:user).permit(:entry_sort, :starred_feed_enabled, :hide_tagged_feeds, :precache_images, :show_unread_count, :sticky_view_inline, :mark_as_read_confirmation, :apple_push_notification_device_token, :receipt_info)
  end

  def get_entry_counts(feed_ids)
    start_date = 29.days.ago
    end_date = Time.now

    max = max_entry_count(feed_ids, start_date)
    stats_query = relative_entry_count_query

    entry_counts = {}
    feed_ids.each do |feed_id|
      query = ActiveRecord::Base.send(:sanitize_sql_array, [stats_query, start_date, end_date, max, start_date, feed_id])
      results = ActiveRecord::Base.connection.execute(query)
      results.each do |result|
        if entry_counts.has_key?(feed_id)
          entry_counts[feed_id] << result['entries_count']
        else
          entry_counts[feed_id] = [result['entries_count']]
        end
      end
    end
    entry_counts
  end

  def max_entry_count(feed_ids, start_date)
    max_query = "SELECT COALESCE(MAX(entries_count), 0) as max FROM feed_stats WHERE feed_id IN(?) and day >= ?"
    max_query = ActiveRecord::Base.send(:sanitize_sql_array, [max_query, feed_ids, start_date])
    max = ActiveRecord::Base.connection.execute(max_query)
    max.first['max'].to_i
  end

  def relative_entry_count_query
    <<-eos
      SELECT
        date,
        coalesce(entries_count,0) AS entries_count
      FROM
      generate_series(
        ?::date,
        ?::date,
        '1 day'
      ) AS date
      LEFT OUTER JOIN (
        SELECT
        day,
        (entries_count::float / ?) AS entries_count
        FROM feed_stats
        WHERE day >= ?
        AND feed_id = ?
      ) results
      ON (date = results.day)
    eos
  end

end
